/* Sistemas Integrados TEKNISA
  Controle de Produo

Nome: 2808_CDPSTPO_BUSCA_CUSTO_RECEITA.SQL

Descrio:
   Funo para buscar o custo da receita do produto de acordo com as variaes  do preo da matria prima

Criao:
25/10/2009 - Welbert G. Moitinha

ALTER TABLE TRABDIFCUSTPROD ADD CDPRECOBASE VARCHAR2(02);
/
*/


CREATE OR REPLACE PROCEDURE BUSCA_CUSTO_RECEITA(P_IDOPCAO     IN VARCHAR2,
                                                P_CDFILIAL    IN VARCHAR2,
                                                P_NRTIPOCUSTO IN NUMBER,
                                                P_CDARVINI    IN VARCHAR2,
                                                P_CDARVFIN    IN VARCHAR2,
                                                P_CDPRECOBASE IN VARCHAR2,
                                                P_SEQTRABPROCF OUT VARCHAR2)
IS
   V_CUSTOATUAL FLOAT;
   V_VRHISCUSTOPROD HISTCUSTOPROD.VRHISCUSTOPROD%TYPE;
   V_VRCALCCUSTO FLOAT;
   V_VRPERDIFCUSPREC PARAMPEDIDO.VRPERDIFCUSPREC%TYPE;
   V_STNUMERO NOVOCODIGO.NRSEQUENCIAL%TYPE;
   V_VRDIVISOR FLOAT;
   V_QTRENDINSU    TCOMPPRO.QTPRODTCOMP%TYPE;
   V_IDCUSTINSPROD PARAMAN.IDCUSTINSPROD%TYPE;
   V_NRCASASRET    PARAMAN.NRCASASRET%TYPE;
   V_VRPERDIFCUSTPRO FLOAT;
BEGIN
   /* BUSCA SEQUENCIAL DA TABELA DE TRABALHO */
   NOVO_CODIGO_SC('TRABDIFCUSTPROD'||P_CDFILIAL, V_STNUMERO, 1);
   V_STNUMERO  :=  SUBSTR( V_STNUMERO,11,10);

   BEGIN
      SELECT NVL(VRPERDIFCUSPREC,0)
        INTO V_VRPERDIFCUSPREC
        FROM PARAMPEDIDO
        WHERE CDFILIAL = P_CDFILIAL;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_VRPERDIFCUSPREC := 0;
   END;

   V_VRPERDIFCUSPREC := V_VRPERDIFCUSPREC/100;

   /*VERIFICA TIPO DE CALCULO
     P_IDOPCAO > 0 - Produto Acabado (baseado no custo de produo)
     P_IDOPCAO > 1 - Produto Acabado (baseado nas matrias primas)
     P_IDOPCAO > 2 - Produto de Venda
   */
   V_CUSTOATUAL      := 0;
   V_VRHISCUSTOPROD  := 0;
   V_VRCALCCUSTO     := 0;
   V_VRDIVISOR       := 0;
   V_VRCALCCUSTO     := 0;
   V_VRPERDIFCUSTPRO := 0;
   IF P_IDOPCAO = '0' THEN /*P_IDOPCAO > 0 - Produto Acabado (baseado no custo de produo)*/
      FOR C_PRODACAB IN (SELECT PF.CDPRODUTO, PR.CDPRODESTO, PR.CDARVPROD, PR.VRFATOCONV
                           FROM PROCFABRICA PF JOIN PRODUTO PR
                                                 ON PF.CDPRODUTO = PR.CDPRODUTO
                          WHERE PF.CDFILIAL = P_CDFILIAL
                            AND PF.IDPROFABPADR = 'S'
                            AND PR.CDARVPROD BETWEEN P_CDARVINI AND P_CDARVFIN||'ZZZZZZZZZZZZZZZZZZZZ'
                            AND PR.CDNVPRODUTO = (SELECT MAX(CDNVPRODUTO) FROM PRODUTO))
      LOOP
         V_CUSTOATUAL := ROUND(CUSTO_PRODUTO(P_CDFILIAL, C_PRODACAB.CDPRODESTO, P_NRTIPOCUSTO, SYSDATE, SYSDATE),2) * C_PRODACAB.VRFATOCONV; /*CUSTO DE ESTOQUE DO PRODUTO ACABADO*/
         V_VRHISCUSTOPROD := ROUND(BUSCA_PRECO_PED(P_CDFILIAL, C_PRODACAB.CDARVPROD, NULL, P_CDPRECOBASE, SYSDATE, NULL), 2); /*PREO DA TABELA*/
         V_VRDIVISOR := V_VRHISCUSTOPROD;
         IF ((V_VRHISCUSTOPROD = 0) OR (V_VRHISCUSTOPROD IS NULL)) THEN
            V_VRDIVISOR := 1;
         END IF;

         V_VRCALCCUSTO :=  ABS( NVL(V_VRHISCUSTOPROD,0) - V_CUSTOATUAL);
         V_VRPERDIFCUSTPRO := (V_VRCALCCUSTO * 100 / NVL(V_VRDIVISOR,1));
         /* TESTA A DIFERENA*/
         IF V_VRPERDIFCUSTPRO >= V_VRPERDIFCUSPREC THEN
            INSERT INTO TRABDIFCUSTPROD( NRSEQDIFCUST, CDFILIALTRAB, CDPRODUTOTRAB, VRCUSTANTPROD, VRCUSTATUPROD, VRPERDIFCUSTPRO, CDPRECOBASE)
                                 VALUES( V_STNUMERO, P_CDFILIAL, C_PRODACAB.CDPRODUTO, V_VRHISCUSTOPROD, V_CUSTOATUAL, V_VRPERDIFCUSTPRO, P_CDPRECOBASE);
         END IF;
      END LOOP;
   ELSE
      IF P_IDOPCAO = '1' THEN /*P_IDOPCAO > 1 - - Produto Acabado (baseado nas matrias primas)*/
         BEGIN
            SELECT NVL(IDCUSTINSPROD,'N'), NVL(NRCASASRET, 6)
              INTO V_IDCUSTINSPROD, V_NRCASASRET
              FROM PARAMAN
             WHERE CDFILIAL = P_CDFILIAL;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN BEGIN
               V_IDCUSTINSPROD := 'N';
               V_NRCASASRET := 6;
            END;
         END;

         FOR C_PRODACAB IN (SELECT PF.CDPRODUTO, PR.CDARVPROD, PF.CDPROCFABRIC
                              FROM PROCFABRICA PF JOIN PRODUTO PR
                                                    ON PF.CDPRODUTO = PR.CDPRODUTO
                             WHERE PF.CDFILIAL = P_CDFILIAL
                               AND PF.IDPROFABPADR = 'S'
                               AND PR.CDARVPROD BETWEEN P_CDARVINI AND P_CDARVFIN||'ZZZZZZZZZZZZZZZZZZZZ'
                               AND PR.CDNVPRODUTO = (SELECT MAX(CDNVPRODUTO) FROM PRODUTO))
         LOOP
            /*CUSTO DO PRODUTO ACABADO BASEADO NO CUSTO DAS MATERIAS PRIMAS*/
            SELECT ROUND(SUM(
                         ROUND(
                         ROUND(QTINSUMETAPA,V_NRCASASRET) *
                         CASE WHEN V_IDCUSTINSPROD = 'N' THEN
                            ROUND(CUSTO_PRODUTO(P_CDFILIAL, CDPRODINSUMO, P_NRTIPOCUSTO, SYSDATE, SYSDATE),2)
                         ELSE
                            CASE WHEN NVL(IDSEMIACETAP, 'N') = 'S' THEN
                               ROUND(CUSTO_SEMIACABADO(P_CDFILIAL,CDPRODINSUMO,P_NRTIPOCUSTO, SYSDATE, SYSDATE,'PRO',NULL),2)
                            ELSE ROUND(CUSTO_PRODUTO_ALMOX(P_CDFILIAL,CDPRODINSUMO,P_NRTIPOCUSTO, SYSDATE, SYSDATE,NULL,NULL),2) END END
                         ,2)),2)
              INTO V_CUSTOATUAL
              FROM INSUMETAPA
             WHERE CDFILIAL  =  P_CDFILIAL
               AND CDPRODUTO =  C_PRODACAB.CDPRODUTO
               AND CDPROCFABRIC = C_PRODACAB.CDPROCFABRIC;

            V_QTRENDINSU := RENDIMENTO_RECEITA_CDP(P_CDFILIAL,C_PRODACAB.CDPRODUTO);
            IF V_QTRENDINSU = 0 THEN
               V_QTRENDINSU := 1;
            END IF;

            V_CUSTOATUAL := ROUND(V_CUSTOATUAL/V_QTRENDINSU,2);

            /*PREO DA TABELA*/
            V_VRHISCUSTOPROD := ROUND(BUSCA_PRECO_PED(P_CDFILIAL, C_PRODACAB.CDARVPROD, NULL, P_CDPRECOBASE, SYSDATE, NULL), 2);
            V_VRDIVISOR := V_VRHISCUSTOPROD;
            IF ((V_VRHISCUSTOPROD = 0) OR (V_VRHISCUSTOPROD IS NULL)) THEN
               V_VRDIVISOR := 1;
            END IF;
            V_VRCALCCUSTO :=  ABS( NVL(V_VRHISCUSTOPROD,0) - V_CUSTOATUAL);
            V_VRPERDIFCUSTPRO := (V_VRCALCCUSTO * 100 / NVL(V_VRDIVISOR,1));
            IF V_VRPERDIFCUSTPRO >= V_VRPERDIFCUSPREC THEN
               INSERT INTO TRABDIFCUSTPROD( NRSEQDIFCUST, CDFILIALTRAB, CDPRODUTOTRAB, VRCUSTANTPROD, VRCUSTATUPROD, VRPERDIFCUSTPRO, CDPRECOBASE)
                                    VALUES( V_STNUMERO, P_CDFILIAL, C_PRODACAB.CDPRODUTO, V_VRHISCUSTOPROD, V_CUSTOATUAL, V_VRPERDIFCUSTPRO, P_CDPRECOBASE);
            END IF;
         END LOOP;
      ELSE
         IF P_IDOPCAO = '3' THEN /*P_IDOPCAO > 2 - Produto de Venda*/
            FOR C_PRODREVENDA IN (SELECT PR.CDPRODUTO, PR.CDPRODESTO, PR.VRFATOCONV, PR.CDARVPROD
                                    FROM PRODUTO PR
                                   WHERE IDPRODPROD <> 'S'
                                     AND PR.CDARVPROD BETWEEN P_CDARVINI AND P_CDARVFIN||'ZZZZZZZZZZZZZZZZZZZZ'
                                     AND PR.CDNVPRODUTO = (SELECT MAX(CDNVPRODUTO) FROM PRODUTO))
            LOOP
               /*CUSTO DE ESTOQUE DO PRODUTO*/
               V_CUSTOATUAL := ROUND(CUSTO_PRODUTO(P_CDFILIAL, C_PRODREVENDA.CDPRODESTO, P_NRTIPOCUSTO, SYSDATE, SYSDATE),2) * C_PRODREVENDA.VRFATOCONV;
               /*PREO DA TABELA*/
               V_VRHISCUSTOPROD := ROUND(BUSCA_PRECO_PED(P_CDFILIAL, C_PRODREVENDA.CDARVPROD, NULL, P_CDPRECOBASE, SYSDATE, NULL), 2);
               V_VRDIVISOR := V_VRHISCUSTOPROD;
               IF ((V_VRHISCUSTOPROD = 0) OR (V_VRHISCUSTOPROD IS NULL)) THEN
                  V_VRDIVISOR := 1;
               END IF;
               V_VRCALCCUSTO :=  ABS( NVL(V_VRHISCUSTOPROD,0) - V_CUSTOATUAL);
               V_VRPERDIFCUSTPRO := (V_VRCALCCUSTO * 100 / NVL(V_VRDIVISOR,1));
               IF V_VRPERDIFCUSTPRO >= V_VRPERDIFCUSPREC THEN
                  INSERT INTO TRABDIFCUSTPROD( NRSEQDIFCUST, CDFILIALTRAB, CDPRODUTOTRAB, VRCUSTANTPROD, VRCUSTATUPROD, VRPERDIFCUSTPRO, CDPRECOBASE)
                                       VALUES( V_STNUMERO, P_CDFILIAL, C_PRODREVENDA.CDPRODUTO, V_VRHISCUSTOPROD, V_CUSTOATUAL, V_VRPERDIFCUSTPRO, P_CDPRECOBASE );
               END IF;
            END LOOP;
         END IF;
      END IF;
   END IF;

   P_SEQTRABPROCF := V_STNUMERO;

END;
/

-- VERSAO --
BEGIN ATUVEROBJ('BUSCA_CUSTO_RECEITA','6.49.059'); END;
/
