package com.javaseleniumtemplate.pages;

import com.javaseleniumtemplate.bases.PageBase;
import org.openqa.selenium.By;
import org.openqa.selenium.Keys;
import org.openqa.selenium.interactions.Actions;
import org.testng.Assert;

public class PadraoCicloPage extends PageBase {

    By botaoMenu = By.xpath("//div[@class='bars']");
    By campoPesquisar = By.xpath("//section[@class='zh-container-search-menu']//input[@type='text']");
    By selecionarOpcaoPadraoCiclo = By.xpath("//section[@class='something-else']//span[text() = 'Padrão de Ciclo']");

    By botaoAdicionar = By.xpath("//div[@class=\"zh-footer-center\"]//div[@class=\"zh-test-box\"]");
    By botaoSalvar = By.xpath("//div[@class=\"zh-footer-right\"]");
    By botaoCancelar = By.xpath("//div[@class=\"zh-footer-left\"]");
    By botaoExcluir = By.xpath("//div[@class=\"zh-footer-center\"]//div[@class=\"zh-test-box\"]");
    By botaoSim = By.xpath("//section[@class=\"zh-container-alert\"]//button[1]");
    By botaoOk = By.xpath("//section[@class=\"zh-container-alert\"]//button[@type=\"submit\"]");
    By botaoDeOpcoesDaTela = By.xpath("//div[@data-ng-if=\"widget.isFloatingControlVisible()\"]//div[@data-zh-floating-control]");
    By botaoPesquisar = By.xpath("//div[@class=\"control-menu\"]//li[@data-zh-id=\"zh-floating-control-custom-action-1\"]");
    By campoProcurarTelaInicial = By.xpath("//li[@class=\"float-action expandable fix-position-bottom opened\"]//input");

    By campoCodigo = By.id("CDPADCICLO");
    By campoNome = By.id("NMPADCICLO");

    By checkBoxPadraoCicloLinha1 = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]/div[1]/div[@data-zh-field-name=\"__CHECKBOX\"]");
    By codigoPadraoCicloLinha1 = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]/div[1]/div[@data-zh-field-name=\"CDPADCICLO\"]/span");
    By nomePadraoCicloLinha1 = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]/div[1]/div[@data-zh-field-name=\"NMPADCICLO\"]/span");

    By mensagemCantoSuperiorDireito = By.xpath("//*[@class=\"zh-notification-messages\"]//span[@class=\"notification-message-text\"]");
    By mensagemAlerta = By.xpath("//section[@class=\"zh-container-alert\"]//div[@class=\"zh-alert-scroll\"]/p");
    By mensagemNenhumRegistroEncontrado = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]//p[@class=\"text-center zh-text-search-not-found ng-binding\"]");
    By mensagemObrigatorioCodigo = By.xpath("//div[@data-zh-id=\"4478476822839877127\"]//div[@class=\"zh-validation\"]/span");
    By mensagemObrigatorioNome = By.xpath("//div[@data-zh-id=\"4478476540327564128\"]//div[@class=\"zh-validation\"]/span");

    public void acessarTela() {
        tempoParaAguardarTelaInicial();
        waitForElement(botaoMenu);
        click(botaoMenu);
        sendKeys(campoPesquisar, "Padrão de Ciclo");
        click(selecionarOpcaoPadraoCiclo);
    }

    public void adicionarPadrao() {
        tempo();
        click(botaoAdicionar);
        sendKeys(campoCodigo, "5");
        sendKeys(campoNome, "AUTOMAÇÃO");
        tempo();
        click(botaoSalvar);

        String mensagem = getText(mensagemCantoSuperiorDireito);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de sucesso, após adicionar o padrão de ciclo");
    }

    public void verificarPadraoCicloAdd() {
        preencherCampoProcurarTelaInicial("00005");
        String codigo = getText(codigoPadraoCicloLinha1);
        String nome = getText(nomePadraoCicloLinha1);
        Assert.assertEquals(codigo, "00005", "Verificar código do padrão ciclo adicionado");
        Assert.assertEquals(nome, "AUTOMAÇÃO", "Verificar nome do padrão ciclo adicionado");
    }

    public void editarPadraoCiclo() {
        preencherCampoProcurarTelaInicial("00002");
        alterarNomeDoPadraoCiclo("PREPARAÇÃO");

        String mensagem = getText(mensagemCantoSuperiorDireito);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem de salvo com sucesso exibida no canto da tela, após alterar o nome do padrão de ciclo");

        tempo();
        String nome = getText(nomePadraoCicloLinha1);
        Assert.assertEquals(nome, "PREPARAÇÃO", "Verificar se o nome alterado foi salvo corretamente");
        refresh();
        preencherCampoProcurarTelaInicial("00002");
        alterarNomeDoPadraoCiclo("COMPRAS");
        tempo();
        nome = getText(nomePadraoCicloLinha1);
        Assert.assertEquals(nome, "COMPRAS", "Verificar se o nome alterado foi salvo corretamente, retornando para o nome que estava no início do teste");
    }

    public void excluirPadraoCiclo() {
        preencherCampoProcurarTelaInicial("00005");
        click(checkBoxPadraoCicloLinha1);
        click(botaoExcluir);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Deletar o(s) registro(s)?";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem exibida na tela após clicar para excluir o padrão de ciclo");
        click(botaoSim);

        mensagem = getText(mensagemCantoSuperiorDireito);
        mensagemEsperada = "Excluído com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem de excluído com sucesso exibida na canto da tela após excluir um padrão de ciclo");

        mensagem = getText(mensagemNenhumRegistroEncontrado);
        mensagemEsperada = "Nenhum registro foi encontrado";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem exibida no grid da lista de padrões de ciclo, informando que nenhum registro foi encontrado");
    }

    public void camposObrigatorios() {
        click(botaoAdicionar);
        click(botaoSalvar);

        String codigo = getText(mensagemObrigatorioCodigo);
        String nome = getText(mensagemObrigatorioNome);
        String mensagemEsperada = "Campo obrigatório";
        Assert.assertEquals(codigo, mensagemEsperada, "Verificar mensagem de campo obrigatório no campo código");
        Assert.assertEquals(nome, mensagemEsperada, "Verificar mensagem de campo obrigatório no campo nome");
        click(botaoCancelar);
    }

    public void tentarAddPadraoComCodigoExistente() {
        click(botaoAdicionar);
        sendKeys(campoCodigo, "1");
        sendKeys(campoNome, "ENTREGA");
        click(botaoSalvar);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Impossível adicionar. Registro já existe.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensaagem exibida na tela após tentar adicionar um padrão de ciclo com código já cadastrado");
        click(botaoOk);
        tempo();
        click(botaoCancelar);
    }

    public void tentarExcluirPadraoComRegFilho() {
        preencherCampoProcurarTelaInicial("00001");
        click(checkBoxPadraoCicloLinha1);
        click(botaoExcluir);
        click(botaoSim);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Exclusão não permitida! O registro possui registros filhos.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem exibida na tela após click no botão excluir um padrão de ciclo que possui registros filhos");
        click(botaoOk);
        tempo();
        click(checkBoxPadraoCicloLinha1);
    }

    private void alterarNomeDoPadraoCiclo(String nome) {
        Actions actions = new Actions(driver);
        tempo();
        actions.click(driver.findElement(nomePadraoCicloLinha1)).build().perform();
        tempo();
        actions.sendKeys(Keys.BACK_SPACE).build().perform();
        tempo();
        actions.sendKeys(nome).build().perform();
        tempo();
        actions.sendKeys(Keys.ENTER).build().perform();
        tempo();
        click(botaoSalvar);
        tempo();
    }

    private void preencherCampoProcurarTelaInicial(String texto) {
        tempo();
        tempo();
        tempo();
        tempo();
        tempo();
        if (returnClassOpened(botaoDeOpcoesDaTela) && !returnClassOpened(botaoPesquisar)) {
            click(botaoPesquisar);
        } else if (!returnClassOpened(botaoDeOpcoesDaTela)){
            click(botaoDeOpcoesDaTela);
            click(botaoPesquisar);
        }
        clearAndSendKeys(campoProcurarTelaInicial, texto);
        tempo();
    }

    private boolean returnClassOpened(By locator) {
        return driver.findElement(locator).getAttribute("className").contains("opened");
    }

}
