package com.javaseleniumtemplate.tests;

import com.javaseleniumtemplate.bases.TestBase;
import com.javaseleniumtemplate.flows.LoginFlows;
import com.javaseleniumtemplate.flows.PontoDeEntregaFlows;
import com.javaseleniumtemplate.pages.PontoDeEntregaPage;
import org.testng.Assert;
import org.testng.annotations.Test;

public class PontoDeEntregaTests extends TestBase {
    PontoDeEntregaFlows pontoDeEntregaFlows;
    PontoDeEntregaPage pontoDeEntregaPage;
    LoginFlows loginFlows;

    public void acessarTela(String acaoTela, String CDFILIAL){
        String msgEsperada = "Ponto de Entrega";
        loginFlows = new LoginFlows();
        pontoDeEntregaFlows = new PontoDeEntregaFlows();
        loginFlows.efetuarLogin();
        pontoDeEntregaFlows.navegaPontoDeEntrega(acaoTela,CDFILIAL);
        Assert.assertEquals(pontoDeEntregaFlows.confirmaAcessoTela(),msgEsperada,"Falha ao acessar tela: ");

    }

    @Test
    public void validarCamposObrigatorios(){
        String MSGESPERADA = "Campo obrigatório";
        String acaoTela = "Validar";
        String CDFILIAL = "0002";
        acessarTela(acaoTela,CDFILIAL);
        pontoDeEntregaFlows.validarCamposObrigatorios();
        Assert.assertEquals(pontoDeEntregaFlows.campoObrigatorioCodigo(),MSGESPERADA);
        Assert.assertEquals(pontoDeEntregaFlows.campoObrigatorioNome(),MSGESPERADA);
    }

    @Test
    public void validarCadastroRegistroExistente(){
        String acaoTela = "Validar";
        String CDFILIAL = "0002";
        acessarTela(acaoTela,CDFILIAL);
        pontoDeEntregaFlows.validarCadastroRegistroExistente();
    }

    @Test
    public void cadastraPontoEntrega(){
    String acaoTela = "Cadastrar";
    String CDFILIAL = "0002";
    acessarTela(acaoTela,CDFILIAL);
    String CDPONTOENTREGA = "050";
    String CDPONTENTREGA = "051";
    String NMPONTOENTREGA = "TEKNISA";
    String ENDEPONTOENTREGA = "RUA PERNAMBUCO 1000";
    String NMPONTENTREGA = "CLIENTE TEKNISA";
    String ENDPONTOENTREGA = "AV. SOLAR DO ENGENHO, 21";

   pontoDeEntregaFlows.cadastrarPontoEntrega(NMPONTOENTREGA,ENDEPONTOENTREGA);

   Assert.assertEquals(pontoDeEntregaFlows.validaCadastroCodigoPontoEntrega(),CDPONTOENTREGA);
   Assert.assertEquals(pontoDeEntregaFlows.validaCadastroNomePontoEntrega(),NMPONTOENTREGA);
   Assert.assertEquals(pontoDeEntregaFlows.validaCadastroEnderecoPontoEntrega(),ENDEPONTOENTREGA);
   Assert.assertEquals(pontoDeEntregaFlows.validarCadastroCodigoPontoEntrega(),CDPONTENTREGA);
   Assert.assertEquals(pontoDeEntregaFlows.validarCadastroNomePontoEntrega(),NMPONTENTREGA);
   Assert.assertEquals(pontoDeEntregaFlows.validarCadastroEnderecoPontoEntrega(),ENDPONTOENTREGA);

    }


    @Test
    public void EditarPontoEntrega(){
        String acaoTela = "Editar";
        String CDFILIAL = "0003";
        String NMPONTOENTREGA = "AUTOMAÇÃO";
        String ENDPONTOENTREGA = "AV. CARLOS SANTOS, 123";
        String NOMEPONTOENTREGA = "AUTOMAÇÃO TESTE";
        String ENDERECOPONTOENTREGA = "AV. PERNAMBUCO, 125";
        String MSGESPERADA_1 = "AUTOMAÇÃO";
        String MSGESPERADA_2 = "AV. CARLOS SANTOS, 123";
        String MSGESPERADA_3 = "AUTOMAÇÃO TESTE";
        String MSGESPERADA_4 = "AV. PERNAMBUCO, 125";

        acessarTela(acaoTela,CDFILIAL);
        pontoDeEntregaFlows.alteraPontoDeEntrega(NMPONTOENTREGA,ENDPONTOENTREGA,NOMEPONTOENTREGA,ENDERECOPONTOENTREGA);
        Assert.assertEquals(pontoDeEntregaFlows.nomeEditado002(),MSGESPERADA_1,"Falha ao alterar o nome do ponto de entrega 002:");
        Assert.assertEquals(pontoDeEntregaFlows.enderecoEditado002(),MSGESPERADA_2,"Falha ao alterar o endereço do ponto de entrega 002:");
        Assert.assertEquals(pontoDeEntregaFlows.nomeEditado003(),MSGESPERADA_3,"Falha ao alterar o nome do ponto de entrega 003:");
        Assert.assertEquals(pontoDeEntregaFlows.enderecoEditado003(),MSGESPERADA_4,"Falha ao alterar o endereço do ponto de entrega 003:");

    }

    @Test
    public void excluirPontosEntrega(){
        String acaoTela = "Excluir";
        String CDFILIAL = "0002";
        String MSGESPERADA = "Excluído com sucesso.";
        acessarTela(acaoTela,CDFILIAL);
        pontoDeEntregaFlows.excluirPontoDeEntrega();
        Assert.assertEquals(pontoDeEntregaFlows.validarExclusao(),MSGESPERADA,"Exclusao ponto de entrega: ");
        Assert.assertEquals(pontoDeEntregaFlows.validarExclusao(),MSGESPERADA,"Exclusao ponto de entrega: ");
    }



}
